#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.

""" Utilities for testing notification system """

# pylint: disable=E0401, E0402

from unittest.mock import MagicMock, patch

import pytest
from mock_alchemy.mocking import UnifiedAlchemyMagicMock
from pyramid.config import Configurator
from pyramid.testing import DummyRequest, setUp, tearDown

from workspaces.notification.schema import NotificationTemplate
from workspaces.notification.services.notification_info import NotificationInfo
from workspaces.notification.services.notification_service import NotificationService


@pytest.fixture(scope="session")
def mock_notification_info() -> NotificationInfo:
    """
    Pytest fixture providing a mock NotificationInfo object

    :return: Mocked NotificationInfo
    """
    email_notification = NotificationTemplate(
        name="email",
        description="A generic email template, requiring subject and message.",
        content="Subject: {{subject}}\n\n{{message}}",
    )

    mock_session = UnifiedAlchemyMagicMock()
    mock_notification_info = NotificationInfo(mock_session)
    mock_session.add(email_notification)
    with patch("workspaces.notification.services.notification_info.NotificationInfo.save_entity"):
        return mock_notification_info


@pytest.fixture(scope="session")
def mock_notification_service(mock_notification_info: NotificationInfo) -> NotificationService:
    """
    Pytest fixture providing a mock NotificationService object

    :return: Mocked NotificationService
    """
    return NotificationService(mock_notification_info)


@pytest.fixture(scope="module")
def test_config() -> Configurator:
    """
    Returns a dummy Configurator object for testing
    :return: Dummy Configurator
    """
    request = DummyRequest()
    config = setUp(request=request)
    config.add_request_method(lambda lookup: MagicMock(), "notification_info", reify=True)
    yield config
    tearDown()


@pytest.fixture(scope="module")
def dummy_send(mock_notification_info, mock_notification_service) -> DummyRequest:
    """
    returns a dummy request with a mocked notification info

    :return: DummyRequest with mock NotificationInfo
    """
    request = DummyRequest(notification_info=mock_notification_info, notification_service=mock_notification_service)
    return request
